@@ -14,7 +14,7 @@ class BidirMap(object):
             self.__setitem__(key, map[key])
 
     def __setitem__(self, key, value):
-        if self.v2k.has_key(value):
+        if value in self.v2k:
             if self.v2k[value] != key:
                 raise KeyError("Value '"+str(value)+"' already in use with key '"+str(self.v2k[value])+"'")
         try:
