@@ -6,6 +6,8 @@
 ## License: GPL Version 2
 ## Copyright: TGRMN Software and contributors
 
+from __future__ import absolute_import, division
+
 import os
 import sys
 import time
@@ -15,7 +17,7 @@ import random
 import errno
 from calendar import timegm
 from logging import debug, warning, error
-from ExitCodes import EX_OSFILE
+from .ExitCodes import EX_OSFILE
 try:
     import dateutil.parser
 except ImportError:
@@ -33,20 +35,26 @@ $ pip install python-dateutil
     sys.stderr.flush()
     sys.exit(EX_OSFILE)
 
-import Config
-import Exceptions
-
-# hashlib backported to python 2.4 / 2.5 is not compatible with hmac!
-if sys.version_info[0] == 2 and sys.version_info[1] < 6:
-    from md5 import md5
-else:
-    from hashlib import md5
+try:
+    from urllib import quote
+except ImportError:
+    # python 3 support
+    from urllib.parse import quote
 
 try:
-    import xml.etree.ElementTree as ET
-except ImportError:
-    # xml.etree.ElementTree was only added in python 2.5
-    import elementtree.ElementTree as ET
+    unicode
+except NameError:
+    # python 3 support
+    # In python 3, unicode -> str, and str -> bytes
+    unicode = str
+
+import S3.Config
+import S3.Exceptions
+import xml.dom.minidom
+
+from hashlib import md5
+
+import xml.etree.ElementTree as ET
 
 __all__ = []
 def parseNodes(nodes):
@@ -70,11 +78,19 @@ def parseNodes(nodes):
     return retval
 __all__.append("parseNodes")
 
+def getPrettyFromXml(xmlstr):
+    xmlparser = xml.dom.minidom.parseString(xmlstr)
+    return xmlparser.toprettyxml()
+
+__all__.append("getPrettyFromXml")
+
+
 def stripNameSpace(xml):
     """
     removeNameSpace(xml) -- remove top-level AWS namespace
+    Operate on raw byte(utf-8) xml string. (Not unicode)
     """
-    r = re.compile('^(<?[^>]+?>\s*)(<\w+) xmlns=[\'"](http://[^\'"]+)[\'"](.*)', re.MULTILINE)
+    r = re.compile(b'^(<?[^>]+?>\s*)(<\w+) xmlns=[\'"](http://[^\'"]+)[\'"](.*)', re.MULTILINE)
     if r.match(xml):
         xmlns = r.match(xml).groups()[2]
         xml = r.sub("\\1\\2\\4", xml)
@@ -90,7 +106,7 @@ def getTreeFromXml(xml):
         if xmlns:
             tree.attrib['xmlns'] = xmlns
         return tree
-    except Exception, e:
+    except Exception as e:
         error("Error parsing xml: %s", e)
         error(xml)
         raise
@@ -112,7 +128,7 @@ def getDictFromTree(tree):
         else:
             content = decode_from_s3(child.text) if child.text is not None else None
         child_tag = decode_from_s3(child.tag)
-        if ret_dict.has_key(child_tag):
+        if child_tag in ret_dict:
             if not type(ret_dict[child_tag]) == list:
                 ret_dict[child_tag] = [ret_dict[child_tag]]
             ret_dict[child_tag].append(content or "")
@@ -190,12 +206,15 @@ def formatDateTime(s3timestamp):
     return date_obj.strftime("%Y-%m-%d %H:%M")
 __all__.append("formatDateTime")
 
-def convertTupleListToDict(list):
+def convertHeaderTupleListToDict(list):
+    """
+    Header keys are always in lowercase in python2 but not in python3.
+    """
     retval = {}
     for tuple in list:
-        retval[tuple[0]] = tuple[1]
+        retval[tuple[0].lower()] = tuple[1]
     return retval
-__all__.append("convertTupleListToDict")
+__all__.append("convertHeaderTupleListToDict")
 
 _rnd_chars = string.ascii_letters+string.digits
 _rnd_chars_len = len(_rnd_chars)
@@ -208,14 +227,14 @@ def rndstr(len):
 __all__.append("rndstr")
 
 def mktmpsomething(prefix, randchars, createfunc):
-    old_umask = os.umask(0077)
+    old_umask = os.umask(0o077)
     tries = 5
     while tries > 0:
         dirname = prefix + rndstr(randchars)
         try:
             createfunc(dirname)
             break
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.EEXIST:
                 os.umask(old_umask)
                 raise
@@ -265,50 +284,68 @@ def mkdir_with_parents(dir_name):
         try:
             debug("mkdir(%s)" % cur_dir)
             os.mkdir(deunicodise(cur_dir))
-        except (OSError, IOError), e:
+        except (OSError, IOError) as e:
             debug("Can not make directory '%s' (Reason: %s)" % (cur_dir, e.strerror))
             return False
-        except Exception, e:
+        except Exception as e:
             debug("Can not make directory '%s' (Reason: %s)" % (cur_dir, e))
             return False
     return True
 __all__.append("mkdir_with_parents")
 
-def unicodise(string, encoding = None, errors = "replace"):
+def unicodise(string, encoding = None, errors = "replace", silent=False):
     """
     Convert 'string' to Unicode or raise an exception.
     """
 
     if not encoding:
-        encoding = Config.Config().encoding
+        encoding = S3.Config.Config().encoding
 
     if type(string) == unicode:
         return string
-    debug("Unicodising %r using %s" % (string, encoding))
+
+    if not silent:
+        debug("Unicodising %r using %s" % (string, encoding))
     try:
         return unicode(string, encoding, errors)
     except UnicodeDecodeError:
         raise UnicodeDecodeError("Conversion to unicode failed: %r" % string)
 __all__.append("unicodise")
 
-def deunicodise(string, encoding = None, errors = "replace"):
+def unicodise_s(string, encoding = None, errors = "replace"):
+    """
+    Alias to silent version of unicodise
+    """
+    return unicodise(string, encoding, errors, True)
+__all__.append("unicodise_s")
+
+def deunicodise(string, encoding = None, errors = "replace", silent=False):
     """
     Convert unicode 'string' to <type str>, by default replacing
     all invalid characters with '?' or raise an exception.
     """
 
     if not encoding:
-        encoding = Config.Config().encoding
+        encoding = S3.Config.Config().encoding
 
     if type(string) != unicode:
-        return str(string)
-    debug("DeUnicodising %r using %s" % (string, encoding))
+        return string
+
+    if not silent:
+        debug("DeUnicodising %r using %s" % (string, encoding))
     try:
         return string.encode(encoding, errors)
     except UnicodeEncodeError:
         raise UnicodeEncodeError("Conversion from unicode failed: %r" % string)
 __all__.append("deunicodise")
 
+def deunicodise_s(string, encoding = None, errors = "replace"):
+    """
+    Alias to silent version of deunicodise
+    """
+    return deunicodise(string, encoding, errors, True)
+__all__.append("deunicodise_s")
+
 def unicodise_safe(string, encoding = None):
     """
     Convert 'string' to Unicode according to current encoding
@@ -338,7 +375,7 @@ def encode_to_s3(string, errors = "replace"):
     all invalid characters with '?' or raise an exception.
     """
     if type(string) != unicode:
-        return str(string)
+        return string
     # Be quiet by default
     #debug("Encoding string to S3: %r" % string)
     try:
@@ -347,6 +384,25 @@ def encode_to_s3(string, errors = "replace"):
         raise UnicodeEncodeError("Conversion from unicode failed: %r" % string)
 __all__.append("encode_to_s3")
 
+## Low level methods
+def urlencode_string(string, urlencoding_mode = None, unicode_output=False):
+    string = encode_to_s3(string)
+
+    if urlencoding_mode is None:
+        urlencoding_mode = S3.Config.Config().urlencoding_mode
+
+    if urlencoding_mode == "verbatim":
+        ## Don't do any pre-processing
+        return string
+
+    encoded = quote(string, safe="~/")
+    debug("String '%s' encoded to '%s'" % (string, encoded))
+    if unicode_output:
+        return decode_from_s3(encoded)
+    else:
+        return encode_to_s3(encoded)
+__all__.append("urlencode_string")
+
 def replace_nonprintables(string):
     """
     replace_nonprintables(string)
@@ -366,7 +422,7 @@ def replace_nonprintables(string):
             modified += 1
         else:
             new_string += c
-    if modified and Config.Config().urlencoding_mode != "fixbucket":
+    if modified and S3.Config.Config().urlencoding_mode != "fixbucket":
         warning("%d non-printable characters replaced in: %s" % (modified, new_string))
     return new_string
 __all__.append("replace_nonprintables")
@@ -387,7 +443,7 @@ def time_to_epoch(t):
     elif hasattr(t, 'strftime'):
         # Looks like the object supports standard srftime()
         return int(t.strftime('%s'))
-    elif isinstance(t, str) or isinstance(t, unicode):
+    elif isinstance(t, str) or isinstance(t, unicode) or isinstance(t, bytes):
         # See if it's a string representation of an epoch
         try:
             # Support relative times (eg. "+60")
@@ -398,45 +454,45 @@ def time_to_epoch(t):
             # Try to parse it as a timestamp string
             try:
                 return time.strptime(t)
-            except ValueError, ex:
+            except ValueError as ex:
                 # Will fall through
                 debug("Failed to parse date with strptime: %s", ex)
                 pass
-    raise Exceptions.ParameterError('Unable to convert %r to an epoch time. Pass an epoch time. Try `date -d \'now + 1 year\' +%%s` (shell) or time.mktime (Python).' % t)
+    raise S3.Exceptions.ParameterError('Unable to convert %r to an epoch time. Pass an epoch time. Try `date -d \'now + 1 year\' +%%s` (shell) or time.mktime (Python).' % t)
 
 
 def check_bucket_name(bucket, dns_strict = True):
     if dns_strict:
         invalid = re.search("([^a-z0-9\.-])", bucket, re.UNICODE)
         if invalid:
-            raise Exceptions.ParameterError("Bucket name '%s' contains disallowed character '%s'. The only supported ones are: lowercase us-ascii letters (a-z), digits (0-9), dot (.) and hyphen (-)." % (bucket, invalid.groups()[0]))
+            raise S3.Exceptions.ParameterError("Bucket name '%s' contains disallowed character '%s'. The only supported ones are: lowercase us-ascii letters (a-z), digits (0-9), dot (.) and hyphen (-)." % (bucket, invalid.groups()[0]))
     else:
         invalid = re.search("([^A-Za-z0-9\._-])", bucket, re.UNICODE)
         if invalid:
-            raise Exceptions.ParameterError("Bucket name '%s' contains disallowed character '%s'. The only supported ones are: us-ascii letters (a-z, A-Z), digits (0-9), dot (.), hyphen (-) and underscore (_)." % (bucket, invalid.groups()[0]))
+            raise S3.Exceptions.ParameterError("Bucket name '%s' contains disallowed character '%s'. The only supported ones are: us-ascii letters (a-z, A-Z), digits (0-9), dot (.), hyphen (-) and underscore (_)." % (bucket, invalid.groups()[0]))
 
     if len(bucket) < 3:
-        raise Exceptions.ParameterError("Bucket name '%s' is too short (min 3 characters)" % bucket)
+        raise S3.Exceptions.ParameterError("Bucket name '%s' is too short (min 3 characters)" % bucket)
     if len(bucket) > 255:
-        raise Exceptions.ParameterError("Bucket name '%s' is too long (max 255 characters)" % bucket)
+        raise S3.Exceptions.ParameterError("Bucket name '%s' is too long (max 255 characters)" % bucket)
     if dns_strict:
         if len(bucket) > 63:
-            raise Exceptions.ParameterError("Bucket name '%s' is too long (max 63 characters)" % bucket)
+            raise S3.Exceptions.ParameterError("Bucket name '%s' is too long (max 63 characters)" % bucket)
         if re.search("-\.", bucket, re.UNICODE):
-            raise Exceptions.ParameterError("Bucket name '%s' must not contain sequence '-.' for DNS compatibility" % bucket)
+            raise S3.Exceptions.ParameterError("Bucket name '%s' must not contain sequence '-.' for DNS compatibility" % bucket)
         if re.search("\.\.", bucket, re.UNICODE):
-            raise Exceptions.ParameterError("Bucket name '%s' must not contain sequence '..' for DNS compatibility" % bucket)
+            raise S3.Exceptions.ParameterError("Bucket name '%s' must not contain sequence '..' for DNS compatibility" % bucket)
         if not re.search("^[0-9a-z]", bucket, re.UNICODE):
-            raise Exceptions.ParameterError("Bucket name '%s' must start with a letter or a digit" % bucket)
+            raise S3.Exceptions.ParameterError("Bucket name '%s' must start with a letter or a digit" % bucket)
         if not re.search("[0-9a-z]$", bucket, re.UNICODE):
-            raise Exceptions.ParameterError("Bucket name '%s' must end with a letter or a digit" % bucket)
+            raise S3.Exceptions.ParameterError("Bucket name '%s' must end with a letter or a digit" % bucket)
     return True
 __all__.append("check_bucket_name")
 
 def check_bucket_name_dns_conformity(bucket):
     try:
         return check_bucket_name(bucket, dns_strict = True)
-    except Exceptions.ParameterError:
+    except S3.Exceptions.ParameterError:
         return False
 __all__.append("check_bucket_name_dns_conformity")
 
@@ -448,10 +504,7 @@ def check_bucket_name_dns_support(bucket_host, bucket_name):
     if "%(bucket)s" not in bucket_host:
         return False
 
-    try:
-        return check_bucket_name(bucket_name, dns_strict = True)
-    except Exceptions.ParameterError:
-        return False
+    return check_bucket_name_dns_conformity(bucket_name)
 __all__.append("check_bucket_name_dns_support")
 
 def getBucketFromHostname(hostname):
@@ -463,17 +516,19 @@ def getBucketFromHostname(hostname):
 
     Returns bucket name and a boolean success flag.
     """
+    if "%(bucket)s" not in S3.Config.Config().host_bucket:
+        return (hostname, False)
 
     # Create RE pattern from Config.host_bucket
-    pattern = Config.Config().host_bucket % { 'bucket' : '(?P<bucket>.*)' }
+    pattern = S3.Config.Config().host_bucket % { 'bucket' : '(?P<bucket>.*)' }
     m = re.match(pattern, hostname, re.UNICODE)
     if not m:
         return (hostname, False)
-    return m.groups()[0], True
+    return m.group(1), True
 __all__.append("getBucketFromHostname")
 
 def getHostnameFromBucket(bucket):
-    return Config.Config().host_bucket % { 'bucket' : bucket }
+    return S3.Config.Config().host_bucket % { 'bucket' : bucket }
 __all__.append("getHostnameFromBucket")
 
 
@@ -500,21 +555,21 @@ try:
     import pwd
     def getpwuid_username(uid):
         """returns a username from the password databse for the given uid"""
-        return pwd.getpwuid(uid).pw_name
+        return unicodise_s(pwd.getpwuid(uid).pw_name)
 except ImportError:
     import getpass
     def getpwuid_username(uid):
-        return getpass.getuser()
+        return unicodise_s(getpass.getuser())
 __all__.append("getpwuid_username")
 
 try:
     import grp
     def getgrgid_grpname(gid):
         """returns a groupname from the group databse for the given gid"""
-        return  grp.getgrgid(gid).gr_name
+        return unicodise_s(grp.getgrgid(gid).gr_name)
 except ImportError:
     def getgrgid_grpname(gid):
-        return "nobody"
+        return u"nobody"
 
 __all__.append("getgrgid_grpname")
 
